package edu.stanford.facs.cytometers;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import com.MeehanMetaSpace.Basics;
import com.MeehanMetaSpace.IoBasics;

import edu.stanford.facs.cytometers.beans.FACS;
import edu.stanford.facs.cytometers.beans.FACSConfiguration;
import edu.stanford.facs.cytometers.beans.LaserConfiguration;
import edu.stanford.facs.cytometers.beans.OpticalDetector;
import edu.stanford.facs.cytometers.beans.OpticalFilter;

public class InstrumentReader {

	public InstrumentReader() {
		// TODO Auto-generated constructor stub
	}

	private BufferedReader getStream(String from) {
		InputStreamReader isr = new InputStreamReader(this.getClass().getClassLoader().getResourceAsStream(from));
		BufferedReader br= new BufferedReader(isr);
		return br;
	}
	
	public static List<String> getInstrumentNames(File inputFile) {
		ArrayList<String> names = new ArrayList<String>();
		ArrayList<String> list = null;
		try {
			if (inputFile == null) {
				list = IoBasics.readTextLinesAndClose(
						new InstrumentReader().getStream("edu/stanford/facs/cytometers/configurations/Instruments.csv"), false);

			} else {
				list = IoBasics.readTextFileLines(inputFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		for (String data : list) {
			String d[] = data.split(",");
			names.add(d[0]);
		}

		return names;
	}
	public static String experimentName = "";
	public static String instrumentName = "";
	public static String species = "";
	
	public static Vector[] getInstrument(File inputFile) throws Exception {
		ArrayList<String> names = new ArrayList<String>();
		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
        Vector<String> columnNames = new Vector<String>();
        
        ArrayList<String> list = null;
		try {
			if (inputFile == null) {
				list = IoBasics.readTextLinesAndClose(
						new InstrumentReader().getStream("edu/stanford/facs/cytometers/configurations/BDAriaIII.csv"), false);

			} else {
				list = IoBasics.readTextFileLines(inputFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		columnNames.add("Laser");
		columnNames.add("Filter");
		columnNames.add("Antigen");
		columnNames.add("Flourochrome");
		int count = 1;
		
		for (String l: list) {
			if (count == 1) {
				experimentName = l;
				if (experimentName.length() > 25 || experimentName.split(",").length > 1) {
					throw new Exception ("Not a valid StainSet configuration file");
				}
				
				count++;
				continue;
			}
			if (count == 2) {
				instrumentName = l;
				if (instrumentName.length() > 15 || instrumentName.split(",").length > 1) {
					throw new Exception ("Not a valid StainSet configuration file");
				}
				
				count++;
				continue;
			}
			if (count == 3) {
				species = l;
				if (species.length() > 10 || species.split(",").length > 1) {
					throw new Exception ("Not a valid StainSet configuration file");
				}
				count++;
				continue;
			}
			if (!Basics.isEmpty(l)) {
				Vector<Object> lineData = new Vector<Object>();
				String d[]=l.split(",");
				if (d.length != 4) {
					throw new Exception ("Not a valid StainSet configuration file");
				}
				lineData.add(d[0]);
				lineData.add(d[1]);
				lineData.add(d[2]);
				lineData.add(d[3]);
				data.add(lineData);
			}
			
		}
		return new Vector[]{data, columnNames};
	}

	public static String[] getDyes(File inputFile) {
		String[] dyeKeys = null;
		Set<String> dyeSet;
		ArrayList<String> list = null;
		try {
			if (inputFile == null) {
				list = IoBasics.readTextLinesAndClose(
						new InstrumentReader().getStream("edu/stanford/facs/cytometers/configurations/Dyes.txt"),
						false);

			} else {
				list = IoBasics.readTextFileLines(inputFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		dyeSet = new TreeSet<String>();
		for (final String s : list) {
			if (!s.startsWith("Hypothetical ")) {
				dyeSet.add(s);
			}
		}
		dyeKeys = dyeSet.toArray(new String[0]);

		return dyeKeys;
	}
	
	public static String[] getMarkers(File inputFile) {
		String[] dyeKeys = null;
		Set<String> dyeSet;
		
		ArrayList<String> list = null;
		try {
			if (inputFile == null) {
				list = IoBasics.readTextLinesAndClose(
						new InstrumentReader().getStream("edu/stanford/facs/cytometers/configurations/Markers.txt"),
						false);

			} else {
				list = IoBasics.readTextFileLines(inputFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		dyeSet = new TreeSet<String>();
		boolean firstList = true;
		for (final String s : list) {
			if (firstList) {
				firstList = false;
				continue;
			}
			String d[] = s.split("\t");
			for (String x: d) {
				dyeSet.add(x);
			}
			
		}
		dyeKeys = dyeSet.toArray(new String[0]);
		return dyeKeys;
	}
	
	
	
	public static FACSConfiguration getFacs(File inputFile) {
		

		ArrayList<String> list = null;
		try {
			if (inputFile == null) {
				list = IoBasics.readTextLinesAndClose(
						new InstrumentReader().getStream("edu/stanford/facs/cytometers/configurations/LSRII.txt"),
						false);

			} else {
				list = IoBasics.readTextFileLines(inputFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		FACSConfiguration facsConfig = new FACSConfiguration();
		FACS facs = new FACS();
		
		try {
			//facs.setModel(model);
			facsConfig.setName(list.get(5));
			facsConfig.setKey(list.get(5));
			facs.setKey(list.get(1));
			facs.setSoftwareType(list.get(2));
			facs.setProducer(list.get(0));
			facs.setMaxLasers(Integer.parseInt(list.get(3)));
			facs.setMaxOpticalDetectors(Integer.parseInt(list.get(4)));
			facsConfig.setFacs(facs);
			boolean laser=false;
			LaserConfiguration laserConfig = null ;
			ArrayList<LaserConfiguration> lasers = new ArrayList<LaserConfiguration>();
			ArrayList<OpticalDetector> ods = new ArrayList<OpticalDetector>();
			for (String data: list) {
				if (data.startsWith("Blue") || 
						data.startsWith("Red") ||
						data.startsWith("Green") ||
						data.startsWith("Violet"))  {
					laserConfig = new LaserConfiguration();
					String d[]=data.split(",");
					laserConfig.setKey(d[0]);
					laserConfig.setWavelength(Integer.parseInt(d[1]));
					laserConfig.setPower(Integer.parseInt(d[2]));
					lasers.add(laserConfig);
					//facsConfig.addLaserConfiguration(laserConfig);
					laser = true;
				}
				else if (laser) {
					OpticalDetector od = new OpticalDetector();
					OpticalFilter filter = new OpticalFilter();
					String d[]=data.split(",");
					filter.setWavelengthCenter(Integer.parseInt(d[0].split("/")[0]));
					filter.setWavelengthRange(Integer.parseInt(d[0].split("/")[1]));
					filter.setKey(d[0]);
					od.setOpticalFilterKey(filter.getKey());
					od.setLaserKey(laserConfig.getKey());
					od.setFcsParameterNumber(Integer.parseInt(d[1]));
					od.setKey(d[2]);
					od.setOpticalDetectorTOBandPass(filter);
					od.setOpticalDetectorTOLaser (laserConfig);
					ods.add(od);
					//facsConfig.addFacsTOOpticalDetector(od);
				}
			}			
			facsConfig.setLaserConfiguration(lasers.toArray(new LaserConfiguration[0]));
			facsConfig.setFacsTOOpticalDetector(ods.toArray(new OpticalDetector[0]));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return facsConfig;
	}
	
	public static FACSConfiguration populateFacs(Vector<Vector<Object>> data) {
		
		FACSConfiguration facsConfig = new FACSConfiguration();
		FACS facs = new FACS();
		facsConfig.setExperimentName(experimentName);
		facsConfig.setName(instrumentName);//"BD Aria III"
		facsConfig.setKey(instrumentName);
		facs.setKey(instrumentName);
		facs.setSoftwareType("Diva Software");
		facs.setProducer("BD");
		facs.setMaxLasers(5);
		facs.setMaxOpticalDetectors(14);
		facsConfig.setFacs(facs);
		LaserConfiguration laserConfig = null ;
		ArrayList<LaserConfiguration> lasers = new ArrayList<LaserConfiguration>();
		ArrayList<OpticalDetector> ods = new ArrayList<OpticalDetector>();
		String laser ="";
		int counter=0;
		if (!Basics.isEmpty(data)) {
			for (Vector<Object> v: data) {
				counter++;
				String newLaser = (String)v.get(0);
				if(Basics.isEmpty(newLaser)) {
					newLaser="L" + counter;
				}
				if (!newLaser.equals(laser)) {
					laser= newLaser;
					laserConfig = new LaserConfiguration();
					String l[] = newLaser.split("-");
					laserConfig.setKey(newLaser);
					int i = 0;
					try {
						i = Integer.parseInt(l[0]);
						laserConfig.setWavelength(i);
						laserConfig.setPower(10);
						
					}catch (Exception e) {}
					
					
					lasers.add(laserConfig);
				}
				//else {
					String filters = (String)v.get(1);
					String d[] = filters.split("/");
					OpticalDetector od = new OpticalDetector();
					OpticalFilter filter = new OpticalFilter();
					try {
						filter.setWavelengthCenter(Integer.parseInt(d[0]));
						filter.setKey(d[0]);
						
					}catch (Exception e) {
						filter.setKey("F"+counter);
					}
					try {
						filter.setWavelengthRange(Integer.parseInt(d[1]));
						
					}catch (Exception e) {}
					
					od.setOpticalFilterKey(filter.getKey());
					od.setLaserKey(laserConfig.getKey());
					//od.setFcsParameterNumber(Integer.parseInt(d[1]));
					od.setKey(filter.getKey());
					od.setAlternativeParameterNames(new String[] {(String)v.get(3)});
					od.setOpticalDetectorTOBandPass(filter);
					od.setOpticalDetectorTOLaser (laserConfig);
					ods.add(od);
				//}
			}
		}
		facsConfig.setLaserConfiguration(lasers.toArray(new LaserConfiguration[0]));
		facsConfig.setFacsTOOpticalDetector(ods.toArray(new OpticalDetector[0]));
		return facsConfig;
	}
	
	public static void main(String[] args) {
		

	}

}
